using System;
using System.Collections;
using Server;
using Server.Items;
using Server.Regions;
using Server.Network;
using Server.ContextMenus;

namespace Server.Items
{
	public abstract class BaseBoard : Container
	{
		public BaseBoard( int itemID, int gumpID, Rectangle2D bounds ) : base( itemID, gumpID, -1, bounds )
		{
			CreatePieces();
		}

		public abstract void CreatePieces();

		public void Reset()
		{
			for ( int i = Items.Count - 1; i >= 0; --i )
				if ( i < Items.Count )
					((Item)Items[i]).Delete();

			CreatePieces();
		}

		public void CreatePiece( BasePiece piece, int x, int y )
		{
			AddItem( piece );
			piece.Location = new Point3D( x, y, 0 );
		}

		public BaseBoard( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );
			writer.Write( (int) 0 );
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );
			int version = reader.ReadInt();
		}

		public override TimeSpan DecayTime{ get{ return TimeSpan.FromDays( 1.0 ); } }

		public override void OnSingleClick( Mobile from )
		{
			if ( Name == null )
				LabelTo( from, 1020000 + ItemID );
			else
				LabelTo( from, Name );
		}

		public override bool OnDragDrop( Mobile from, Item dropped )
		{
			BasePiece piece = dropped as BasePiece;

			return ( piece != null && piece.Board == this && base.OnDragDrop( from, dropped ) );
		}

		public override bool OnDragDropInto( Mobile from, Item dropped, Point3D point )
		{
			BasePiece piece = dropped as BasePiece;

			if ( piece != null && piece.Board == this && base.OnDragDropInto( from, dropped, point ) )
			{
				Packet p = new PlaySound( 0x127, GetWorldLocation() );

				if ( RootParent == from )
				{
					from.Send( p );
				}
				else
				{
					Map map = Map;

					if ( map != null )
					{
						IPooledEnumerable eable = map.GetClientsInRange( GetWorldLocation(), 2 );

						foreach ( NetState state in eable )
							state.Send( p );

						eable.Free();
					}
				}

				return true;
			}
			else
			{
				return false;
			}
		}

		public override void GetContextMenuEntries( Mobile from, ArrayList list )
		{
			base.GetContextMenuEntries( from, list );

			if ( ValidateDefault( from, this ) )
				list.Add( new DefaultEntry( from, this ) );
		}

		public static bool ValidateDefault( Mobile from, BaseBoard board )
		{
			if ( from.AccessLevel >= AccessLevel.GameMaster )
				return true;

			if ( board.IsChildOf( from.Backpack ) )
				return true;

			object root = board.RootParent;

			if ( root is Mobile && root != from )
				return false;

			if ( board.Deleted || board.Map != from.Map || !from.InRange( board.GetWorldLocation(), 1 ) )
				return false;

			Region r = Region.Find( board.GetWorldLocation(), board.Map );

			return ( r is HouseRegion && ((HouseRegion)r).House != null && ((HouseRegion)r).House.Owner == from );
		}

		public class DefaultEntry : ContextMenuEntry
		{
			private Mobile m_From;
			private BaseBoard m_Board;

			public DefaultEntry( Mobile from, BaseBoard board ) : base( 94, from.AccessLevel >= AccessLevel.GameMaster ? -1 : 1 )
			{
				m_From = from;
				m_Board = board;
			}

			public override void OnClick()
			{
				if ( BaseBoard.ValidateDefault( m_From, m_Board ) )
					m_Board.Reset();
			}
		}
	}
}