using System;
using Server;

namespace Server.Misc
{
	public class SkillCheck
	{
		public static void Initialize()
		{
			Mobile.SkillCheckHandler = new SkillCheckHandler( Mobile_SkillCheck );
			Mobile.SkillCheckDirectHandler = new SkillCheckDirectHandler( Mobile_SkillCheckDirect );
		}

		public static bool Mobile_SkillCheck( Mobile from, SkillName skillName, double minSkill, double maxSkill )
		{
			Skill skill = from.Skills[skillName];

			if ( skill == null )
				return false;

			double value = skill.Value;

			if ( value < minSkill )
				return false; // Too difficult
			else if ( value >= maxSkill )
				return true; // No challenge

			double chance = (value - minSkill) / (maxSkill - minSkill);

			return CheckSkill( from, skill, chance );
		}

		public static bool Mobile_SkillCheckDirect( Mobile from, SkillName skillName, double chance )
		{
			Skill skill = from.Skills[skillName];

			if ( skill == null )
				return false;

			if ( chance < 0.0 )
				return false; // Too difficult
			else if ( chance >= 1.0 )
				return true; // No challenge

			return CheckSkill( from,  skill, chance );
		}

		public static bool CheckSkill( Mobile from, Skill skill, double chance )
		{
			bool success = ( chance >= Utility.RandomDouble() );
			double gc = ( from.Skills.Cap - from.Skills.Total ) / from.Skills.Cap;
			gc += ( skill.Cap - skill.Base ) / skill.Cap;
			gc /= 2;

			gc += ( 1.0 - chance ) * ( success ? 0.5 : 0.2 );
			gc /= 2;

			gc *= skill.Info.GainFactor;

			if ( gc < 0.01 )
				gc = 0.01;

			if ( skill.Base < 10.0 )
				gc = 1.0;

			if ( gc >= Utility.RandomDouble() && from.Alive )
				Gain( from, skill );

			return success;
		}

		public enum Stat { Str, Dex, Int }

		public static void Gain( Mobile from, Skill skill )
		{
			if ( skill.Base < skill.Cap && skill.Lock == SkillLock.Up )
			{
				Skills skills = from.Skills;

				if ( ( skills.Total / skills.Cap ) >= Utility.RandomDouble() )//( skills.Total >= skills.Cap )
				{
					for ( int i = 0; i < skills.Length; ++i )
					{
						Skill toLower = skills[i];

						if ( toLower != skill && toLower.Lock == SkillLock.Down && toLower.BaseFixedPoint > 0 )
						{
							--toLower.BaseFixedPoint;
							break;
						}
					}
				}

				if ( skills.Total < skills.Cap )
				{
					if ( skill.Base <= 10.0 )
					{
						int give = Utility.Random( 4 ) + 1;
						if ( give/10.0 > skills.Cap - skills.Total )
							give = (int)((skills.Cap - skills.Total)*10.0);
						skill.BaseFixedPoint += give;
					}
					else
					{
						++skill.BaseFixedPoint;
					}

					SkillInfo info = skill.Info;

					if ( from.StrLock == StatLockType.Up && (info.StrGain / 33.3) >= Utility.RandomDouble() )
						GainStat( from, Stat.Str );
					else if ( from.DexLock == StatLockType.Up && (info.DexGain / 33.3) >= Utility.RandomDouble() )
						GainStat( from, Stat.Dex );
					else if ( from.IntLock == StatLockType.Up && (info.IntGain / 33.3) >= Utility.RandomDouble() )
						GainStat( from, Stat.Int );
				}
			}
		}

		public static bool CanLower( Mobile from, Stat stat )
		{
			switch ( stat )
			{
				case Stat.Str: return ( from.StrLock == StatLockType.Down && from.RawStr > 10 );
				case Stat.Dex: return ( from.DexLock == StatLockType.Down && from.RawDex > 10 );
				case Stat.Int: return ( from.IntLock == StatLockType.Down && from.RawInt > 10 );
			}

			return false;
		}

		public static bool CanRaise( Mobile from, Stat stat )
		{
			if ( from.RawStatTotal >= from.StatCap )
				return false;

			switch ( stat )
			{
				case Stat.Str: return ( from.StrLock == StatLockType.Up && from.RawStr < 125 );
				case Stat.Dex: return ( from.DexLock == StatLockType.Up && from.RawDex < 125 );
				case Stat.Int: return ( from.IntLock == StatLockType.Up && from.RawInt < 125 );
			}

			return false;
		}

		public static void IncreaseStat( Mobile from, Stat stat, bool atrophy )
		{
			atrophy = atrophy || (from.RawStatTotal >= from.StatCap);

			switch ( stat )
			{
				case Stat.Str:
				{
					if ( atrophy )
					{
						if ( CanLower( from, Stat.Dex ) && (from.RawDex < from.RawInt || !CanLower( from, Stat.Int )) )
							--from.RawDex;
						else if ( CanLower( from, Stat.Int ) )
							--from.RawInt;
					}

					if ( CanRaise( from, Stat.Str ) )
						++from.RawStr;

					break;
				}
				case Stat.Dex:
				{
					if ( atrophy )
					{
						if ( CanLower( from, Stat.Str ) && (from.RawStr < from.RawInt || !CanLower( from, Stat.Int )) )
							--from.RawStr;
						else if ( CanLower( from, Stat.Int ) )
							--from.RawInt;
					}

					if ( CanRaise( from, Stat.Dex ) )
						++from.RawDex;

					break;
				}
				case Stat.Int:
				{
					if ( atrophy )
					{
						if ( CanLower( from, Stat.Str ) && (from.RawStr < from.RawDex || !CanLower( from, Stat.Dex )) )
							--from.RawStr;
						else if ( CanLower( from, Stat.Dex ) )
							--from.RawDex;
					}

					if ( CanRaise( from, Stat.Int ) )
						++from.RawInt;

					break;
				}
			}
		}

		public static void GainStat( Mobile from, Stat stat )
		{
			bool atrophy = ( (from.RawStatTotal / (double)from.StatCap) >= Utility.RandomDouble() );

			IncreaseStat( from, stat, atrophy );
		}
	}
}