using System;
using System.Collections;
using Server;
using Server.Items;

namespace Server.Multis
{
	public class SmallBoat : Boat
	{
		public SmallBoat( Mobile owner ) : base( owner, 0x4000 )
		{
		}

		public SmallBoat( Serial serial ) : base(serial)
		{
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize(reader);

			int version = reader.ReadInt();
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize(writer);

			writer.Write( (int)0 );
		}

		public override int Width {	get { return 4;	} }
		public override int Height { get { return 12; } }

		public override Point3D PlankOffset { get { return new Point3D( 2, 0, 0 ); } }
		public override Point3D TillerOffset { get { return new Point3D( 0, 4, 0 );	} }
		public override Point3D HoldOffset { get {	return new Point3D( 0, -4, 0 );	} }
	}


	public abstract class Boat : BaseMulti
	{
		//The Z level of water on your map, OSI default is -5
		public const int WaterLevel = -5;
		public const int BridgeHeight = 15;//minimum distance of a bridge from the waterlevel (this closer to the water than this will block boats)

		private static readonly Point2D WorldEdge = new Point2D( 5110, 4086 );//where we loop back to the other side of the map

		private Plank m_SPlank;
		private Plank m_PPlank;
		private TillerMan m_Tiller;
		private Hold m_Hold;
		private Mobile m_Owner;
		private int m_BaseID;
		private Timer m_MoveTimer;

		public Boat( Mobile owner, int baseid ) : base( baseid | 0x4000 )
		{
			m_BaseID = baseid | 0x4000;
			m_Owner = owner;
			Movable = false;

			Container pack = owner.Backpack;
			uint keyval = Key.RandomValue();
			Key key = new Key( KeyType.Rusty, keyval );
			if ( pack == null || !pack.TryDropItem( owner, key, false ) )
				key.MoveToWorld( owner.Location, owner.Map );
			owner.BankBox.DropItem( new Key( KeyType.Rusty, keyval ) );

			m_SPlank = new Plank( this, PlankSide.Starboard, keyval );
			m_PPlank = new Plank( this, PlankSide.Port, keyval );
			m_Tiller = new TillerMan( this );
			m_Hold = new Hold( this );

			m_PPlank.MoveToWorld( new Point3D( -PlankOffset.X, PlankOffset.Y, PlankOffset.Z ), this.Map );
			m_SPlank.MoveToWorld( PlankOffset, this.Map );
			m_Tiller.MoveToWorld( new Point3D( 1+TillerOffset.X, TillerOffset.Y, TillerOffset.Z ), this.Map );
			m_Hold.MoveToWorld( HoldOffset, this.Map );
		}

		public abstract int Width { get; }
		public abstract int Height{ get; }
		public abstract Point3D PlankOffset{ get; }
		public abstract Point3D HoldOffset{ get; }
		public abstract Point3D TillerOffset{ get; }

		public Boat( Serial serial ) : base(serial)
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int)0 );//version
			writer.Write( m_Owner );
			writer.Write( m_PPlank );
			writer.Write( m_SPlank );
			writer.Write( m_Tiller );
			writer.Write( m_Hold );
			writer.Write( m_BaseID );
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();

			switch ( version )
			{
				case 0:
				{
					m_Owner = reader.ReadMobile();
					m_PPlank = reader.ReadItem() as Plank;
					m_SPlank = reader.ReadItem() as Plank;
					m_Tiller = reader.ReadItem() as TillerMan;
					m_Hold = reader.ReadItem() as Hold;
					m_BaseID = reader.ReadInt();

					break;
				}
			}
		}

		public override void OnLocationChange( Point3D old )
		{
			m_PPlank.Location = new Point3D( this.X + m_PPlank.X - old.X, this.Y + m_PPlank.Y - old.Y, this.Z + m_PPlank.Z - old.Z );
			m_SPlank.Location = new Point3D( this.X + m_SPlank.X - old.X, this.Y + m_SPlank.Y - old.Y, this.Z + m_SPlank.Z - old.Z );
			m_Tiller.Location = new Point3D( this.X + m_Tiller.X - old.X, this.Y + m_Tiller.Y - old.Y, this.Z + m_Tiller.Z - old.Z );
			m_Hold.Location = new Point3D( this.X + m_Hold.X - old.X, this.Y + m_Hold.Y - old.Y, this.Z + m_Hold.Z - old.Z );
		}

		public override void OnMapChange()
		{
			m_PPlank.Map = this.Map;
			m_SPlank.Map = this.Map;
			m_Tiller.Map = this.Map;
			m_Hold.Map = this.Map;
		}

		public override void OnAfterDelete()
		{
			if ( m_Tiller != null )
				m_Tiller.Delete();
			if ( m_SPlank != null )
				m_SPlank.Delete();
			if ( m_PPlank != null )
				m_PPlank.Delete();
			if ( m_Hold != null )
				m_Hold.Delete();
		}

		public virtual TimeSpan MoveDelay {	get	{ return TimeSpan.FromSeconds( 0.5 ); } }

		public void StartMove( int speed )
		{
			StartMove( this.Direction, speed );
		}

		public void StartMove( Direction dir, int speed )
		{
			StopMove();

			m_MoveTimer = new MoveTimer( this, dir, speed );
			m_MoveTimer.Start();
		}

		public void StopMove()
		{
			if ( m_MoveTimer != null && m_MoveTimer.Running )
				m_MoveTimer.Stop();
		}

		public bool Move( int dist )
		{
			return Move( this.Direction, dist );
		}

		public static bool CheckBlock( Rectangle2D block, Map map )
		{
			int range = block.Width/2;
			if ( block.Height/2 > range )
				range = block.Height/2;
			++range;

			IPooledEnumerable eable = map.GetObjectsInRange( new Point3D( block.X + block.Width/2, block.Y+block.Height/2, -5 ), range );
			foreach ( object o in eable )
			{
				if ( o is Item )
				{
					Item item = (Item)o;

					int top = item.Z + item.ItemData.Height;

					if ( block.Contains( item.Location ) && 
						top >= WaterLevel && top <= WaterLevel+BridgeHeight && 
						item.Z >= WaterLevel && item.Z <= WaterLevel+BridgeHeight )
					{
						eable.Free();
						Console.WriteLine( "Boat blocked by ITEM at {0} (Top: {1})", item.Location, top );
						return false;
					}
				}
				else if ( o is Mobile )
				{
					Mobile mob = (Mobile)o;
					if ( block.Contains( mob.Location ) && mob.Z < WaterLevel+BridgeHeight && mob.Z > WaterLevel-BridgeHeight )
					{
						eable.Free();
						return false;
					}
				}
			}

			eable.Free();

			for (int x=block.Start.X;x<=block.End.X;x++)
			{
				for (int y=block.Start.Y;y<=block.End.Y;y++)
				{
					bool staticWater = false;
					Tile[] statics = map.Tiles.GetStaticTiles( x, y, true );
					for ( int i=0;i<statics.Length;i++ )
					{
						int top = statics[i].Height + statics[i].Z;
						int z = statics[i].Z;

						if ( (TileData.ItemTable[statics[i].ID&0x3FFF].Flags&TileFlag.Wet) != 0 && z == WaterLevel )
						{
							staticWater = true;
						}
						else if ( ( z >= WaterLevel && z <= WaterLevel+BridgeHeight ) || ( top >= WaterLevel && top <= WaterLevel+BridgeHeight ) )
						{
							Console.WriteLine( "Blocked by STATIC at {0}, {1}, {2} (t:{3}) (f:{4})", x, y, z, top, (int)TileData.ItemTable[statics[i].ID&0x3FFF].Flags );
							return false;
						}
					}

					if ( !staticWater )
					{
						Tile land = map.Tiles.GetLandTile( x, y );
						if ( ( TileData.LandTable[land.ID&0x3FFF].Flags & TileFlag.Wet ) == 0 && land.Z >= WaterLevel )
						{
							Console.WriteLine( "Boat Blocked by LAND! z:{0} f:{1}", land.Z, (int)TileData.LandTable[land.ID&0x3FFF].Flags );
							return false;
						}
					}
				}
			}

			return true;
		}

		public bool Move( Direction dir, int distance )
		{
			//TODO: Blocking
			int ivdir = (int)dir + 4;
			if ( ivdir > 7 )
				ivdir %= 8;

			int x=0,y=0,w=0,h=0;
			if ( dir == this.Direction || ((Direction)ivdir) == this.Direction )
			{//forward or backward
				switch ( dir )
				{
					case Direction.North:
						x = this.X - Width/2;
						y = this.Y - Height/2 - distance;
						w = Width;
						h = distance;
						break;
					case Direction.East:
						x = this.X + Height/2 + distance;
						y = this.Y - Width/2;
						w = distance;
						h = Width;
						break;
					case Direction.South:
						x = this.X - Width/2;
						y = this.Y + Height/2 + distance;
						w = Width;
						h = distance;
						break;
					case Direction.West:
						x = this.X - Height/2 - distance;
						y = this.Y - Width/2;
						w = distance;
						h = Width;
						break;
				}
			}
			else
			{//broadside
				switch ( dir )
				{
					case Direction.North:
						x = this.X - Height/2;
						y = this.Y - Width/2 - distance;
						w = Height;
						h = distance-1;
						break;
					case Direction.East:
						x = this.X + Width/2 + distance;
						y = this.Y - Height/2;
						w = distance-1;
						h = Height;
						break;
					case Direction.South:
						x = this.X - Height/2;
						y = this.Y + Width/2 + distance;
						w = Height;
						h = distance-1;
						break;
					case Direction.West:
						x = this.X - Width/2 - distance;
						y = this.Y - Height/2;
						w = distance-1;
						h = Height;
						break;
				}
			}

			//construct a rectangle that extends in front of the boat, check everything inside the rect
			if ( !CheckBlock( new Rectangle2D( x, y, w, h ), this.Map ) )
			{
				m_Tiller.PublicOverheadMessage( Network.MessageType.Regular, 0x00, false, "Arr, there's something in the way!" );
				return false;
			}

			ArrayList move = new ArrayList();
			IPooledEnumerable eable = this.Map.GetObjectsInRange( this.Location, 8 );

			foreach ( object o in eable )
			{
				if ( o is Item )
				{
					Item item  = (Item)o;
					if ( !( item == m_PPlank || item == m_SPlank || item == m_Tiller || item == m_Hold || item == this ) && this.Contains( item ) )
						move.Add( item );
				}
				else if ( o is Mobile )
				{
					Mobile m = (Mobile)o;
					if ( this.Contains( m ) )
						move.Add( m );
				}
			}

			eable.Free();

			int cx=0, cy=0;
			Movement.Movement.Offset( dir, ref cx, ref cy );
			cx *= distance;
			cy *= distance;

			//if they are moving off the map, wrap them around...
			if ( this.X >= WorldEdge.X && cx > 0 )
				cx = -WorldEdge.X + 11;
			else if ( this.X <= 10 && cx < 0 )
				cx = WorldEdge.X - this.X;

			if ( this.Y >= WorldEdge.Y && cy > 0 )
				cy = -WorldEdge.Y + 11;
			else if ( this.Y <= 10 && cy < 0 )
				cy = WorldEdge.X - this.X;
			
			this.Location = new Point3D( this.X + cx, this.Y + cy, this.Z );
			foreach ( object o in move )
			{
				if ( o is Item )
				{
					Item item  = (Item)o;
					item.Location = new Point3D( item.X + cx, item.Y + cy, item.Z );
				}
				else if ( o is Mobile )
				{
					Mobile m = (Mobile)o;
					m.Location = new Point3D( m.X + cx, m.Y + cy, m.Z );
				}
			}//foreach

			return true;
		}

		public bool SetDirection( Direction dir )
		{
			int change = (int)dir - (int)this.Direction;
			if ( change > 4 )
				change = -2;
			else if ( change < -4 )
				change = 2;

			if ( change == 0 )
				return true;

			if ( Math.Abs( change ) == 2 )
			{//no blocking if they are turning around?
				int tx=0,ty=0,bx=0,by=0;
				int w=0,h=0;
				switch ( dir )
				{
					case Direction.North:
						goto case Direction.South;
					case Direction.South:
						tx = this.X - Width/2;
						ty = this.Y - Height/2 - 1;
						bx = tx;
						by = this.Y + Width/2 + 1;
						w = Width;
						h = Height/2 - Width/2 - 1;
						break;
					case Direction.East:
						goto case Direction.West;
					case Direction.West:
						tx = this.X - Height/2 - 1;
						ty = this.Y - Width/2;
						bx = this.X + Width/2 + 1;
						by = ty;
						w = Height/2 - Width/2 - 1;
						h = Width;
						break;
				}

				if ( !CheckBlock( new Rectangle2D( tx, ty, w, h ), this.Map ) || !CheckBlock( new Rectangle2D( bx, by, w, h ), this.Map ) )
				{
					m_Tiller.PublicOverheadMessage( Network.MessageType.Regular, 0x00, false, "Arr, there's something in the way!" );
					return false;
				}
			}

			if ( Direction == Direction.North )//was north, now changing
				m_Tiller.Location = new Point3D( m_Tiller.X - 1, m_Tiller.Y, m_Tiller.Z );

			ItemID = m_BaseID + ((int)dir)/2;
			Direction = dir;

			m_PPlank.Turn( dir );
			m_SPlank.Turn( dir );
			m_Tiller.Turn( dir );
			m_Hold.Turn( dir );

			ArrayList turn = new ArrayList();
			IPooledEnumerable eable = this.Map.GetObjectsInRange( this.Location, 8 );

			foreach ( object o in eable )
			{
				if ( o is Item )
				{
					Item item = (Item)o;
					if ( item == m_PPlank || item == m_SPlank || item == m_Tiller || item == m_Hold || this.Contains( item ) )
						turn.Add( item );
				}
				else if ( o is Mobile )
				{
					Mobile m = (Mobile)o;
					if ( this.Contains( m ) )
						turn.Add( m );
				}
			}//foreach

			eable.Free();

			foreach ( object o in turn )
			{
				if ( o is Item )
				{
					Item item = (Item)o;
					int dx = item.X - this.X;
					int dy = item.Y - this.Y;

					switch ( change )
					{
						case 2:
							item.Location = new Point3D( this.X - dy, this.Y + dx, item.Z );
							break;

						case -2:
							item.Location = new Point3D( this.X + dy, this.Y - dx, item.Z );
							break;

						case 4:
						case -4:
							item.Location = new Point3D( this.X - dx, this.Y - dy, item.Z );
							break;
					}
				}
				else if ( o is Mobile )
				{
					Mobile m = (Mobile)o;
					int dx = m.X - this.X;
					int dy = m.Y - this.Y;

					switch ( change )
					{
						case 2:
							m.Location = new Point3D( this.X - dy, this.Y + dx, m.Z );
							break;

						case -2:
							m.Location = new Point3D( this.X + dy, this.Y - dx, m.Z );
							break;

						case 4:
						case -4:
							m.Location = new Point3D( this.X - dx, this.Y - dy, m.Z );
							break;
					}
				}
			}

			//now becoming north
			if ( dir == Direction.North )
				m_Tiller.Location = new Point3D( m_Tiller.X + 1, m_Tiller.Y, m_Tiller.Z );

			return true;
		}

		public TillerMan Tiller
		{
			get
			{
				return m_Tiller;
			}
		}

		private class MoveTimer : Timer
		{
			private Boat m_Boat;
			private Direction m_Dir;
			private int m_Dist;

			public MoveTimer( Boat boat, Direction dir, int dist ) : base( TimeSpan.FromSeconds( 0 ), TimeSpan.FromSeconds( 1.0 ) )
			{
				Priority = TimerPriority.TwoFiftyMS;
				m_Boat = boat;
				m_Dir = dir;
				m_Dist = dist;
				Interval = m_Boat.MoveDelay;
			}

			protected override void OnTick()
			{
				if ( !m_Boat.Move( m_Dir, m_Dist ) )
					Stop();
			}
		}
	}
}
